@@ -3,8 +3,8 @@
 #include <iostream>
 #include <RcppArmadillo.h>
 #include <Rcpp.h>
-//#include <boost/math/special_functions/bessel.hpp>
-//#include <boost/math/special_functions/gamma.hpp>
+#include <boost/math/special_functions/bessel.hpp>
+#include <boost/math/special_functions/gamma.hpp>
 
 using namespace Rcpp;
 using namespace arma;
